/////////////////////////////////////////////////////////////////////////////////////////////////
//SENIOR RESEARCH PROJECT - JASON PAK AND DAVID CHA
/////////////////////////////////////////////////////////////////////////////////////////////////

#include <Wire.h>
#include <Adafruit_PWMServoDriver.h>

//Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();
Adafruit_PWMServoDriver pwm1 = Adafruit_PWMServoDriver(0x40);
Adafruit_PWMServoDriver pwm2 = Adafruit_PWMServoDriver(0x41);

/////////////////////////////////////////////////////////////////////////////////////////////////
//NOTE CLASS - special class written for simpler coding 
//Each "Note" represents on key on the xylophone
/////////////////////////////////////////////f////////////////////////////////////////////////////

class Note {
private:
 int servonum; // 0 - 16 on PCA9685
 int dynamic; // 2 - forte, 4 - mf/mp, 6 - piano
 int minval;  //servo value at which mallet touches key
 int maxval; //servo value to which mallet comes up to after touching key
 int driverval; //servo driver number

public:
 Note(int a, int b, int c, int d) {   
 this -> servonum = a;
 this->dynamic = b;
 this->minval = c;
 this->maxval = c + (60 / dynamic); //adjusts maxvalue depending on dynamic level
 this->driverval = d; 
}
 
void Note::playNote(int dur) //plays the note for given duration (millisec)
{
  int curval = this->minval;
  
  if(this->dynamic < 3)
  curval -= 6;
  if(this->dynamic < 5 && this->dynamic > 2)
  curval -= 3;

  if(driverval == 1)
  {
    for (uint16_t pulselen = this->maxval; pulselen > curval; pulselen--) {
      pwm1.setPWM(this->servonum, 0, pulselen);
      delay(this->dynamic);
    }

    for (uint16_t pulselen = curval; pulselen < this->maxval; pulselen++) {
      pwm1.setPWM(this->servonum, 0, pulselen);
      delay(this->dynamic);
    }
  }

  if(driverval == 2)
  {
    for (uint16_t pulselen = this->maxval; pulselen > curval; pulselen--) {
      pwm2.setPWM(this->servonum, 0, pulselen);
      delay(this->dynamic);
    }

    for (uint16_t pulselen = curval; pulselen < this->maxval; pulselen++) {
      pwm2.setPWM(this->servonum, 0, pulselen);
      delay(this->dynamic);
    }
  }

  delay(dur);
 return;
}

int Note::getMin()
{
 return minval;
}

int Note::getMax()
{
 return maxval;
}

int Note::getServo()
{
 return servonum;
}

int Note::getDynamic()
{
 return dynamic;
}

int Note::getDriver()
{
 return driverval;
}

void Note::setDynamic(int x) //when updating the dynamic, the maxval adjusts accordingly as well
{
  this->dynamic = x;
  this->maxval = minval + (60 / x);

  /*
  int curval = this->minval;
  
  if(this->dynamic < 3)
  curval -= 6;
  if(this->dynamic < 5 && this->dynamic > 2)
  curval -= 3;
  
  int totalval = 6 * (this->maxval - this->minval);
  int diff = (totalval/this->dynamic) + curval - maxval;
  this->maxval = maxval + diff; */
}

};

/////////////////////////////////////////////////////////////////////////////////////////////////
//MAIN CODE - code written to perform a tune
/////////////////////////////////////////////////////////////////////////////////////////////////

#define SERVO_FREQ 50 // Analog servos run at ~50 Hz updates

//Instantiate Notes - (servo num, dynamic, min, drivernumber)
Note g1 = Note(0, 2, 109, 1);
Note gs1 = Note(1, 2, 160, 1);
Note a1 = Note(2, 2, 134, 1);
Note as1 = Note(3, 2, 133, 1);
Note b1 = Note(4, 2, 132, 1);
Note c1 = Note(5, 2, 151, 1);
Note cs1 = Note(6, 2, 162, 1);
Note d1 = Note(7, 2, 150, 1);
Note ds1 = Note(8, 2, 122, 1);
Note e1 = Note(9, 2, 120, 1);
Note f1 = Note(10, 2, 140, 1);
Note fs1 = Note(11, 2, 152, 1);
Note g2 = Note(12, 2, 124, 1);
Note gs2 = Note(13, 2, 145, 1); 
Note a2 = Note(14, 2, 100, 1);
Note as2 = Note(15, 2, 166, 1);
Note b2 = Note(0, 2, 178, 2);
Note c2 = Note(1, 2, 162, 2);
Note cs2 = Note(2, 2, 184, 2);
Note d2 = Note(3, 2, 135, 2);
Note ds2 = Note(4, 2, 176, 2);
Note e2 = Note(5, 2, 125, 2);
Note f2 = Note(6, 2, 198, 2);
Note fs2 = Note(7, 2, 165, 2);
Note g3 = Note(8, 2, 160, 2);
Note gs3 = Note(9, 2, 189, 2);
Note a3 = Note(10, 2, 156, 2);
Note as3 = Note(11, 2, 190, 2);
Note b3 = Note(12, 2, 100, 2);
Note c3 = Note(13, 2, 163, 2); 

int temp = 0;

void setup() {
  Serial.begin(9600);
  pwm1.begin();
  //pwm1.setOscillatorFrequency(27000000); 
  pwm1.setPWMFreq(SERVO_FREQ);
  pwm2.begin();
  pwm2.setPWMFreq(SERVO_FREQ);
  //pwm2.setOscillatorFrequency(27000000); 
  delay(10);
}

void loop() {

  //reset dynamic back to forte (2) after reaching piano (6)

  if(g1.getDynamic() > 7) 
  {
    g1.setDynamic(2);
    gs1.setDynamic(2);
    a1.setDynamic(2); 
    as1.setDynamic(2); 
    b1.setDynamic(2); 
    c1.setDynamic(2); 
    cs1.setDynamic(2); 
    d1.setDynamic(2); 
    ds1.setDynamic(2); 
    e1.setDynamic(2); 
    f1.setDynamic(2); 
    fs1.setDynamic(2); 
    g2.setDynamic(2); 
    gs2.setDynamic(2); 
    a2.setDynamic(2); 
    as2.setDynamic(2);
    b2.setDynamic(2);
    c2.setDynamic(2);
    cs2.setDynamic(2);
    d2.setDynamic(2);
    ds2.setDynamic(2);
    e2.setDynamic(2);
    f2.setDynamic(2);
    fs2.setDynamic(2);
    g3.setDynamic(2);
    gs3.setDynamic(2);
    a3.setDynamic(2);
    as3.setDynamic(2);
    b3.setDynamic(2);
    c3.setDynamic(2);
  }

    g1.playNote(50);
    gs1.playNote(50);
    a1.playNote(50); 
    as1.playNote(50); 
    b1.playNote(50); 
    c1.playNote(50); 
    cs1.playNote(50); 
    d1.playNote(50); 
    ds1.playNote(50); 
    e1.playNote(50); 
    f1.playNote(50); 
    fs1.playNote(50); 
    g2.playNote(50); 
    gs2.playNote(50); 
    a2.playNote(50); 
    as2.playNote(50);
    b2.playNote(50);
    c2.playNote(50);
    cs2.playNote(50);
    d2.playNote(50);
    ds2.playNote(50);
    e2.playNote(50);
    f2.playNote(50);
    fs2.playNote(50);
    g3.playNote(50);
    gs3.playNote(50);
    a3.playNote(50);
    as3.playNote(50);
    b3.playNote(50);
    c3.playNote(50);

  delay(1000); 

  g1.setDynamic(g1.getDynamic() + 2); 
  gs1.setDynamic(gs1.getDynamic() + 2); 
  a1.setDynamic(a1.getDynamic() + 2); 
  as1.setDynamic(as1.getDynamic() + 2);  
  b1.setDynamic(b1.getDynamic() + 2); 
  c1.setDynamic(c1.getDynamic() + 2); 
  cs1.setDynamic(cs1.getDynamic() + 2); 
  d1.setDynamic(d1.getDynamic() + 2); 
  ds1.setDynamic(ds1.getDynamic() + 2); 
  e1.setDynamic(e1.getDynamic() + 2); 
  f1.setDynamic(f1.getDynamic() + 2); 
  fs1.setDynamic(fs1.getDynamic() + 2); 
  g2.setDynamic(g2.getDynamic() + 2); 
  gs2.setDynamic(gs2.getDynamic() + 2); 
  a2.setDynamic(a2.getDynamic() + 2); 
  as2.setDynamic(as2.getDynamic() + 2);
  b2.setDynamic(b2.getDynamic() + 2);
  c2.setDynamic(c2.getDynamic() + 2);
  cs2.setDynamic(cs2.getDynamic() + 2);
  d2.setDynamic(d2.getDynamic() + 2);
  ds2.setDynamic(ds2.getDynamic() + 2);
  e2.setDynamic(e2.getDynamic() + 2);
  f2.setDynamic(f2.getDynamic() + 2);
  fs2.setDynamic(fs2.getDynamic() + 2);
  g3.setDynamic(fs2.getDynamic() + 2);
  gs3.setDynamic(fs2.getDynamic() + 2);
  a3.setDynamic(fs2.getDynamic() + 2);
  as3.setDynamic(fs2.getDynamic() + 2);
  b3.setDynamic(fs2.getDynamic() + 2);
  c3.setDynamic(fs2.getDynamic() + 2);

} 

//Method to perform a chord: playing two notes at the same time
void playChord(Note a, Note b, int dur) { 

  int max1 = a.getMax();
  int min1 = a.getMin();
  int diff = b.getMin() - a.getMin();

  for (uint16_t pulselen = max1; pulselen > min1; pulselen--) {
    if(a.getDriver() == 1)    
    pwm1.setPWM(a.getServo(), 0, pulselen);
    if(a.getDriver() == 2)
    pwm2.setPWM(a.getServo(), 0, pulselen);

    if(b.getDriver() == 1)
    pwm1.setPWM(b.getServo(), 0, pulselen + diff);
    if(b.getDriver() == 2)
    pwm2.setPWM(b.getServo(), 0, pulselen + diff);
    
    delay(a.getDynamic());
  }

  for (uint16_t pulselen = min1; pulselen < max1; pulselen++) {
    if(a.getDriver() == 1)    
    pwm1.setPWM(a.getServo(), 0, pulselen);
    if(a.getDriver() == 2)    
    pwm2.setPWM(a.getServo(), 0, pulselen);

    if(b.getDriver() == 1) 
    pwm1.setPWM(b.getServo(), 0, pulselen + diff);
    if(b.getDriver() == 2) 
    pwm1.setPWM(b.getServo(), 0, pulselen + diff);
    
    delay(a.getDynamic());
  }

  delay(dur);
}
